import { defineConfig } from '@agile-ts/utils';
import { Persistent } from '../storages/persistent.js';
import { getSharedStorageManager } from '../storages/shared.js';

typeof require !== "undefined" ? require : (x) => {
  throw new Error('Dynamic require of "' + x + '" is not supported');
};
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const _StatePersistent = class extends Persistent {
  constructor(state, config = {}) {
    super(state.agileInstance(), {
      loadValue: false
    });
    config = defineConfig(config, {
      loadValue: true,
      storageKeys: [],
      defaultStorageKey: null
    });
    this.state = () => state;
    this.instantiatePersistent({
      key: config.key,
      storageKeys: config.storageKeys,
      defaultStorageKey: config.defaultStorageKey
    });
    if (this.ready && config.loadValue)
      this.initialLoading();
  }
  initialLoading() {
    var __super = (key) => super[key];
    return __async(this, null, function* () {
      __super("initialLoading").call(this).then(() => {
        this.state().isPersisted = true;
      });
    });
  }
  loadPersistedValue(storageItemKey) {
    return __async(this, null, function* () {
      var _a;
      if (!this.ready)
        return false;
      const _storageItemKey = storageItemKey != null ? storageItemKey : this._key;
      const loadedValue = yield (_a = getSharedStorageManager()) == null ? void 0 : _a.get(_storageItemKey, this.config.defaultStorageKey);
      if (loadedValue == null)
        return false;
      this.state().set(loadedValue, {
        storage: false,
        overwrite: true
      });
      this.setupSideEffects(_storageItemKey);
      return true;
    });
  }
  persistValue(storageItemKey) {
    return __async(this, null, function* () {
      if (!this.ready)
        return false;
      const _storageItemKey = storageItemKey != null ? storageItemKey : this._key;
      this.setupSideEffects(_storageItemKey);
      this.rebuildStorageSideEffect(this.state(), _storageItemKey);
      this.isPersisted = true;
      return true;
    });
  }
  setupSideEffects(storageItemKey) {
    const _storageItemKey = storageItemKey != null ? storageItemKey : this._key;
    this.state().addSideEffect(_StatePersistent.storeValueSideEffectKey, (instance, config) => {
      this.rebuildStorageSideEffect(this.state(), _storageItemKey, config);
    }, { weight: 0 });
  }
  removePersistedValue(storageItemKey) {
    return __async(this, null, function* () {
      var _a;
      if (!this.ready)
        return false;
      const _storageItemKey = storageItemKey || this._key;
      this.state().removeSideEffect(_StatePersistent.storeValueSideEffectKey);
      (_a = getSharedStorageManager()) == null ? void 0 : _a.remove(_storageItemKey, this.storageKeys);
      this.isPersisted = false;
      return true;
    });
  }
  formatKey(key) {
    if (key == null && this.state()._key)
      return this.state()._key;
    if (key == null)
      return;
    if (this.state()._key == null)
      this.state()._key = key;
    return key;
  }
  rebuildStorageSideEffect(state, storageItemKey, config = {}) {
    var _a;
    if (config["storage"] == null || config.storage) {
      (_a = getSharedStorageManager()) == null ? void 0 : _a.set(storageItemKey, this.state().getPersistableValue(), this.storageKeys);
    }
  }
};
let StatePersistent = _StatePersistent;
StatePersistent.storeValueSideEffectKey = "rebuildStateStorageValue";

export { StatePersistent };
