import { defineConfig, copy } from '@agile-ts/utils';
import { LogCodeManager } from '../logCodeManager.js';
import { getSharedStorageManager } from './shared.js';

typeof require !== "undefined" ? require : (x) => {
  throw new Error('Dynamic require of "' + x + '" is not supported');
};
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const _Persistent = class {
  constructor(agileInstance, config = {}) {
    this.ready = false;
    this.isPersisted = false;
    this.storageKeys = [];
    this.agileInstance = () => agileInstance;
    this._key = _Persistent.placeHolderKey;
    config = defineConfig(config, {
      loadValue: true,
      storageKeys: [],
      defaultStorageKey: null
    });
    this.config = { defaultStorageKey: config.defaultStorageKey };
    if (config.loadValue) {
      this.instantiatePersistent({
        storageKeys: config.storageKeys,
        key: config.key,
        defaultStorageKey: config.defaultStorageKey
      });
    }
  }
  set key(value) {
    this.setKey(value);
  }
  get key() {
    return this._key;
  }
  setKey(value) {
    return __async(this, null, function* () {
      const oldKey = this._key;
      const wasReady = this.ready;
      if (value === this._key)
        return;
      this._key = value != null ? value : _Persistent.placeHolderKey;
      const isValid = this.validatePersistent();
      if (!wasReady) {
        if (isValid)
          yield this.initialLoading();
        return;
      }
      yield this.removePersistedValue(oldKey);
      if (isValid)
        yield this.persistValue(value);
    });
  }
  instantiatePersistent(config = {}) {
    var _a;
    this._key = (_a = this.formatKey(config.key)) != null ? _a : _Persistent.placeHolderKey;
    this.assignStorageKeys(config.storageKeys, config.defaultStorageKey);
    this.validatePersistent();
    const storageManager = getSharedStorageManager();
    if (this._key !== _Persistent.placeHolderKey && storageManager != null) {
      storageManager.persistentInstances[this._key] = this;
    }
  }
  validatePersistent() {
    let isValid = true;
    if (this._key === _Persistent.placeHolderKey) {
      LogCodeManager.log("12:03:00");
      isValid = false;
    }
    if (this.config.defaultStorageKey == null || this.storageKeys.length <= 0) {
      LogCodeManager.log("12:03:01");
      isValid = false;
    }
    this.storageKeys.map((key) => {
      var _a;
      if (!((_a = getSharedStorageManager()) == null ? void 0 : _a.storages[key])) {
        LogCodeManager.log("12:03:02", [this._key, key]);
        isValid = false;
      }
    });
    this.ready = isValid;
    return isValid;
  }
  assignStorageKeys(storageKeys = [], defaultStorageKey) {
    var _a, _b;
    const _storageKeys = copy(storageKeys);
    if (defaultStorageKey != null && !_storageKeys.includes(defaultStorageKey))
      _storageKeys.push(defaultStorageKey);
    if (_storageKeys.length <= 0) {
      const defaultStorageKey2 = (_a = getSharedStorageManager()) == null ? void 0 : _a.config.defaultStorageKey;
      if (defaultStorageKey2 != null) {
        this.config.defaultStorageKey = defaultStorageKey2;
        _storageKeys.push((_b = getSharedStorageManager()) == null ? void 0 : _b.config.defaultStorageKey);
      }
    } else {
      this.config.defaultStorageKey = defaultStorageKey != null ? defaultStorageKey : _storageKeys[0];
    }
    this.storageKeys = _storageKeys;
  }
  initialLoading() {
    return __async(this, null, function* () {
      const success = yield this.loadPersistedValue();
      if (this.onLoad)
        this.onLoad(success);
      if (!success)
        yield this.persistValue();
    });
  }
  loadPersistedValue(storageItemKey) {
    return __async(this, null, function* () {
      LogCodeManager.log("00:03:00", ["loadPersistedValue", "Persistent"]);
      return false;
    });
  }
  persistValue(storageItemKey) {
    return __async(this, null, function* () {
      LogCodeManager.log("00:03:00", ["persistValue", "Persistent"]);
      return false;
    });
  }
  removePersistedValue(storageItemKey) {
    return __async(this, null, function* () {
      LogCodeManager.log("00:03:00", ["removePersistedValue", "Persistent"]);
      return false;
    });
  }
  formatKey(key) {
    return key;
  }
};
let Persistent = _Persistent;
Persistent.placeHolderKey = "__THIS_IS_A_PLACEHOLDER__";

export { Persistent };
